@@ -22,10 +22,14 @@ class ClusterTemplate(base.Resource):
 
 class ClusterTemplateManager(base.ResourceManager):
     resource_class = ClusterTemplate
+    NotUpdated = base.NotUpdated()
+
+    def create(self, name, plugin_name, hadoop_version, description=None,
+               cluster_configs=None, node_groups=None, anti_affinity=None,
+               net_id=None, default_image_id=None, use_autoconfig=None,
+               shares=None, is_public=None, is_protected=None):
+        """Create a Cluster Template."""
 
-    def _assign_field(self, name, plugin_name, hadoop_version,
-                      description=None, cluster_configs=None, node_groups=None,
-                      anti_affinity=None, net_id=None, default_image_id=None):
         data = {
             'name': name,
             'plugin_name': plugin_name,
@@ -38,34 +42,51 @@ class ClusterTemplateManager(base.ResourceManager):
                               node_groups=node_groups,
                               anti_affinity=anti_affinity,
                               neutron_management_network=net_id,
-                              default_image_id=default_image_id)
-        return data
-
-    def create(self, name, plugin_name, hadoop_version, description=None,
-               cluster_configs=None, node_groups=None, anti_affinity=None,
-               net_id=None, default_image_id=None):
-        data = self._assign_field(name, plugin_name, hadoop_version,
-                                  description, cluster_configs, node_groups,
-                                  anti_affinity, net_id, default_image_id)
+                              default_image_id=default_image_id,
+                              use_autoconfig=use_autoconfig,
+                              shares=shares,
+                              is_public=is_public,
+                              is_protected=is_protected)
 
         return self._create('/cluster-templates', data, 'cluster_template')
 
-    def update(self, cluster_template_id, name, plugin_name, hadoop_version,
-               description=None, cluster_configs=None, node_groups=None,
-               anti_affinity=None, net_id=None, default_image_id=None):
-        data = self._assign_field(name, plugin_name, hadoop_version,
-                                  description, cluster_configs, node_groups,
-                                  anti_affinity, net_id, default_image_id)
+    def update(self, cluster_template_id, name=NotUpdated,
+               plugin_name=NotUpdated, hadoop_version=NotUpdated,
+               description=NotUpdated, cluster_configs=NotUpdated,
+               node_groups=NotUpdated, anti_affinity=NotUpdated,
+               net_id=NotUpdated, default_image_id=NotUpdated,
+               use_autoconfig=NotUpdated, shares=NotUpdated,
+               is_public=NotUpdated, is_protected=NotUpdated):
+        """Update a Cluster Template."""
+
+        data = {}
+        self._copy_if_updated(data, name=name,
+                              plugin_name=plugin_name,
+                              hadoop_version=hadoop_version,
+                              description=description,
+                              cluster_configs=cluster_configs,
+                              node_groups=node_groups,
+                              anti_affinity=anti_affinity,
+                              neutron_management_network=net_id,
+                              default_image_id=default_image_id,
+                              use_autoconfig=use_autoconfig,
+                              shares=shares,
+                              is_public=is_public,
+                              is_protected=is_protected)
 
         return self._update('/cluster-templates/%s' % cluster_template_id,
                             data, 'cluster_template')
 
-    def list(self):
-        return self._list('/cluster-templates', 'cluster_templates')
+    def list(self, search_opts=None):
+        """Get list of Cluster Templates."""
+        query = base.get_query_string(search_opts)
+        return self._list('/cluster-templates%s' % query, 'cluster_templates')
 
     def get(self, cluster_template_id):
+        """Get information about a Cluster Template."""
         return self._get('/cluster-templates/%s' % cluster_template_id,
                          'cluster_template')
 
     def delete(self, cluster_template_id):
+        """Delete a Cluster Template."""
         self._delete('/cluster-templates/%s' % cluster_template_id)
